<?php

namespace App\Console\Commands;

use App\Vehicle;
use Illuminate\Console\Command;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;

class PruneVehicleInventory extends Command
{

    private $days;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'vehicles:prune {days}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Prunes vehicles that have been sold more than X days ago.';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $this->days = $this->argument('days');
        $output = $this->output;

        $vehicles = DB::table('wp_posts')
            ->join('wp_postmeta', 'ID', '=', 'post_id', 'inner')
            ->where('post_type', '=', 'vehicle')
            ->where('meta_key', '=', 'delivery_date')
            ->where('meta_value', '<>', '')
            ->get();

        $this->info('Deleting old vehicles...');
        $bar = $output->createProgressBar(count($vehicles));

        foreach ($vehicles as $vehicle) {
            if ($this->isValidDateTimeString($vehicle->meta_value, 'Y-n-d H:i:s', 'America/Toronto')) {
                $delivery_date = Carbon::createFromFormat('Y-n-d H:i:s', $vehicle->meta_value);

                if ($delivery_date <= Carbon::now()->subDays($this->days)) {
                    $vehicle = new Vehicle((array) $vehicle);
                    $vehicle->delete();
                }
            }

            $bar->advance();
        }

        $this->info('');
        $this->info('Completed!');
    }

    private function isValidDateTimeString($str_dt, $str_dateformat, $str_timezone)
    {
        $date = \DateTime::createFromFormat($str_dateformat, $str_dt, new \DateTimeZone($str_timezone));
        return $date && \DateTime::getLastErrors()["warning_count"] == 0 && \DateTime::getLastErrors()["error_count"] == 0;
    }
}
