<?php

namespace App\Events;

use App\Vehicle;
use Illuminate\Broadcasting\Channel;
use Illuminate\Queue\SerializesModels;
use Illuminate\Broadcasting\PrivateChannel;
use Illuminate\Broadcasting\PresenceChannel;
use Illuminate\Foundation\Events\Dispatchable;
use Illuminate\Broadcasting\InteractsWithSockets;
use Illuminate\Contracts\Broadcasting\ShouldBroadcastNow;

class InventoryUpdated implements ShouldBroadcastNow
{
    use Dispatchable, InteractsWithSockets, SerializesModels;

    public $vehicle;


    /**
     * Create a new event instance.
     *
     * @param Vehicle $vehicle
     */
    public function __construct(Vehicle $vehicle)
    {
        $this->vehicle = $vehicle;
        $this->vehicle->status = 'available';

        if ($this->vehicle->is_approved && $this->vehicle->is_sold && !$this->vehicle->is_shipped)
        {
            $this->vehicle->status = 'approved';
        }
        if (!$this->vehicle->is_approved && $this->vehicle->is_sold && !$this->vehicle->is_shipped)
        {
            $this->vehicle->status = 'waiting';
        }
        if ($this->vehicle->is_shipped)
        {
            $this->vehicle->status = 'sold';
        }
    }


    /**
     * Get the channels the event should broadcast on.
     *
     * @return \Illuminate\Broadcasting\Channel|array
     */
    public function broadcastOn()
    {
        return new Channel('map-inventory');
    }
}
