<?php

namespace App;

use App\Events\InventoryUpdated;
use Illuminate\Support\Facades\DB;
use Pusher\Pusher;

/**
 * Copyright (c) Control Web Inc.
 * All rights reserved.
 */

class Vehicle
{
    const CARPROOF_ENDPOINT = "https://webservice.carproof.com/memberreports/lookup.aspx";
    const CARPROOF_ID = "U56BJZ5wqURuxJwTbBrc0Q==";

    private $table_prefix;
    public $fields;
    public $ID;
    public $ID2;

    public function __construct(array $row)
    {
        $this->table_prefix = getenv('DB_PREFIX') ? getenv('DB_PREFIX') : 'wp_';

        $this->fields = [
            'category',
            'year',
            'brand',
            'model',
            'color',
            'mileage',
            'price',
            'pdsf',
            'esthetic',
            'market_price',
            'pdsf_saving',
            'market_saving',
            'engine',
            'gears',
            'gears_description',
            'traction',
            'mpg_city',
            'mpg_highway',
            'nb_passengers',
            'is_sold',
            'is_shipped',
            'stock_nb',
            'is_approved',
            'sug',
            'trm',
            'ecart',
            'equipment',
            'serial_number',
            'approbation_date',
            'delivery_date',
            'pm_approx',
            'location',
            'transmission',
            'mags',
            'ac',
            'awd',
            'sunroof',
            'leather',
        ];

        if (isset($row['ID'])) {
            $this->ID = $row['ID'];
        } else {
            foreach ($this->fields as $index => $field_name) {
                dump($row[$index]);
                $this->$field_name = (strpos($field_name, 'is_') === 0 ? ($row[$index] == 1) : utf8_encode($row[$index]));
            }
        }
    }


    /**
     * Saves the vehicle to the WordPress database
     *
     * @return string $stock_nb
     */
    public function save()
    {
        $vehicle = DB::table($this->table_prefix . 'posts')
            ->join($this->table_prefix . 'postmeta', $this->table_prefix . 'posts.ID', '=', $this->table_prefix . 'postmeta.post_id')
            ->where($this->table_prefix . 'posts.post_type', '=', 'vehicle')
            ->where($this->table_prefix . 'posts.post_name', '=', $this->remove_accents("{$this->brand} {$this->removePlusSign($this->model)} {$this->year} {$this->stock_nb}"))
            ->select($this->table_prefix . 'posts.*')
            ->first();

        $date = date('Y-m-d H:i:s');
        $gmdate = gmdate('Y-m-d H:i:s');

        if (!$vehicle) {

            $this->ID = DB::table($this->table_prefix . 'posts')
                ->insertGetId([
                    'post_author' => 1,
                    'post_date' => $date,
                    'post_date_gmt' => $gmdate,
                    'post_content' => '',
                    'post_title' => "{$this->brand} {$this->model} {$this->year} {$this->stock_nb}",
                    'post_excerpt' => "",
                    'post_status' => "publish",
                    'comment_status' => "closed",
                    'ping_status' => "closed",
                    'post_password' => "",
                    'post_name' => $this->remove_accents("{$this->brand} {$this->removePlusSign($this->model)} {$this->year} {$this->stock_nb}"),
                    'to_ping' => "",
                    'pinged' => "",
                    'post_modified' => $date,
                    'post_modified_gmt' => $gmdate,
                    'post_content_filtered' => '',
                    'post_parent' => 0,
                    'guid' => "",
                    'menu_order' => 0,
                    'post_type' => "vehicle",
                    'post_mime_type' => "",
                    'comment_count' => 0
                ]);
            DB::table($this->table_prefix . 'posts')
                ->where('ID', '=', $this->ID)
                ->update([
                    'guid' => "https://stage.mtlautoprix.com/?post_type=vehicle&p={$this->ID}"
                ]);

            $this->ID2 = DB::table($this->table_prefix . 'posts')
                ->insertGetId([
                    'post_author' => 1,
                    'post_date' => $date,
                    'post_date_gmt' => $gmdate,
                    'post_content' => '',
                    'post_title' => "{$this->brand} {$this->model} {$this->year} {$this->stock_nb}",
                    'post_excerpt' => "",
                    'post_status' => "publish",
                    'comment_status' => "closed",
                    'ping_status' => "closed",
                    'post_password' => "",
                    'post_name' => $this->remove_accents("{$this->brand} {$this->removePlusSign($this->model)} {$this->year} {$this->stock_nb}") . "-en",
                    'to_ping' => "",
                    'pinged' => "",
                    'post_modified' => $date,
                    'post_modified_gmt' => $gmdate,
                    'post_content_filtered' => '',
                    'post_parent' => 0,
                    'guid' => "",
                    'menu_order' => 0,
                    'post_type' => "vehicle",
                    'post_mime_type' => "",
                    'comment_count' => 0
                ]);
            DB::table($this->table_prefix . 'posts')
                ->where('ID', '=', $this->ID2)
                ->update([
                    'guid' => "https://stage.mtlautoprix.com/?post_type=vehicle&p={$this->ID2}"
                ]);

            $this->broadcast();
            // $this->save_carproof_url();
            $this->save_taxonomy();
            $maxtrid = DB::table($this->table_prefix . 'icl_translations')
            ->select($this->table_prefix . 'icl_translations.trid')
            ->orderBy('trid', 'desc')->first();
            $newtrid = 1 + intval($maxtrid->trid);
            DB::table($this->table_prefix . 'icl_translations')
            ->insert([
                'element_id' => $this->ID,
                'element_type' => 'post_vehicle',
                'language_code' => 'fr',
                'trid' => $newtrid,
                'source_language_code' => 'en',
            ]);
            DB::table($this->table_prefix . 'icl_translations')
            ->insert([
                'element_id' => $this->ID2,
                'element_type' => 'post_vehicle',
                'language_code' => 'en',
                'trid' => $newtrid,
                'source_language_code' => 'en',
            ]);
        } else {
            $this->ID = $vehicle->ID;
            $this->ID2 = DB::table($this->table_prefix . 'posts')
                ->join($this->table_prefix . 'postmeta', $this->table_prefix . 'posts.ID', '=', $this->table_prefix . 'postmeta.post_id')
                ->where($this->table_prefix . 'posts.post_type', '=', 'vehicle')
                ->where($this->table_prefix . 'posts.post_name', '=', $this->remove_accents("{$this->brand} {$this->removePlusSign($this->model)} {$this->year} {$this->stock_nb}") . "-en")
                ->select($this->table_prefix . 'posts.ID')
                ->first()
                ->ID;
        }
        foreach ($this->fields as $meta_key) {
            $this->save_meta($meta_key, $this->$meta_key);
        }
        return $this->stock_nb;
    }

    public function delete()
    {
        $return = true;
        foreach ([$this->ID, $this->ID2] as $id) {
            if (isset($id) && $return) {
                DB::table($this->table_prefix . 'posts')
                    ->where('ID', '=', $id)
                    ->delete();

                DB::table($this->table_prefix . 'postmeta')
                    ->where('post_id', '=', $id)
                    ->delete();

                DB::table($this->table_prefix . 'term_relationships')
                    ->where('object_id', '=', $id)
                    ->delete();

                $term_taxonomy_ids = DB::table($this->table_prefix . 'term_relationships')
                    ->where('object_id', '=', $id)
                    ->pluck('term_taxonomy_id')->toArray();

                DB::table($this->table_prefix . 'term_relationships')
                    ->where('object_id', '=', $id)
                    ->delete();

                foreach ($term_taxonomy_ids as $term_taxonomy_id) {
                    $term_id = DB::table($this->table_prefix . 'term_taxonomy')
                        ->where('taxonomy', '=', 'post_translations')
                        ->where('term_taxonomy_id', '=', $term_taxonomy_id)
                        ->pluck('term_id')->first();

                    DB::table($this->table_prefix . 'term_taxonomy')
                        ->where('taxonomy', '=', 'post_translations')
                        ->where('term_taxonomy_id', '=', $term_taxonomy_id)
                        ->delete();

                    DB::table($this->table_prefix . 'terms')
                        ->where('term_id', '=', $term_id)
                        ->delete();
                }
            } else {
                $return = false;
            }
        }
        return $return;
    }


    /**
     * Saves the WordPress postmeta
     *
     * @param string $key
     * @param $value
     */
    private function save_meta(string $key, $value)
    {
        foreach ([$this->ID, $this->ID2] as $id) {
            $updated = false;

            if (isset($id)) {
                $meta = DB::table($this->table_prefix . 'postmeta')
                    ->where('post_id', '=', $id)
                    ->where('meta_key', '=', $key)
                    ->first();

                if (isset($meta->meta_value) && $meta->meta_value != $value) {
                    DB::table($this->table_prefix . 'postmeta')
                        ->where('post_id', '=', $id)
                        ->where('meta_key', '=', $key)
                        ->update([
                            'meta_value' => $value
                        ]);
                    $updated = true;
                } elseif ($meta === NULL) {
                    DB::table($this->table_prefix . 'postmeta')
                        ->insert([
                            'post_id' => $id,
                            'meta_key' => $key,
                            'meta_value' => $value
                        ]);
                }

                // UPDATE REFERENCE KEY
                // $userId = "contact@mtlautoprix.com";
                // $objectId = DB::table($this->table_prefix . 'postmeta')
                //     ->where('post_id', '=', $id)
                //     ->where('meta_key', '=', 'stock_nb')
                //     ->first();
                // $objectIdValue = $objectId->meta_value;
                // $marketplace = "MTL_AUTOPRIX";
                // $reference = hash('sha512',  $userId."_".$objectIdValue."_".$marketplace);
                // $urlrefresh = 'https://api.stampyt.io/v1/panorama-shootings/exists?ref=' . $reference;
                // // $argsrefresh = array(
                // //     'method' => 'GET',
                // //     'headers' => array('x-api-key'=>'lPlap6kxtY1I7DVwp1TNn9J7XgQb41fv8gI2jDcH'),
                // // );
                // // $WP_Http = new WP_Http();
                // // $getresponse = $WP_Http->post($urlrefresh, $argsrefresh);
                // // $getcontent = json_decode( $getresponse['body']);
                // $argsrefresh = array('Content-Type: application/x-www-form-urlencoded', 'charset: utf-8', 'x-api-key: lPlap6kxtY1I7DVwp1TNn9J7XgQb41fv8gI2jDcH');
                // $ch = curl_init();
                // curl_setopt($ch, CURLOPT_URL, $urlrefresh);
                // curl_setopt($ch, CURLOPT_HTTPHEADER, $argsrefresh);
                // curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                // curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
                // curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                // $getcontent = curl_exec($ch);
                // if (curl_errno($ch)) {
                //     echo 'Error:' . curl_error($ch);
                // }
                // curl_close($ch);
                // if( !empty($getcontent) ){
                //     $ref_key = DB::table($this->table_prefix . 'postmeta')
                //     ->where('post_id', '=', $id)
                //     ->where('meta_key', '=', 'ref_key')
                //     ->first()->meta_value;
                //     if( isset($ref_key) && $ref_key !== '' ){
                //         DB::table($this->table_prefix . 'postmeta')
                //         ->where('post_id', '=', $id)
                //         ->where('meta_key', '=', 'ref_key')
                //         ->update([
                //             'meta_value' => $ref_key
                //         ]);
                //     }else{
                //         DB::table($this->table_prefix . 'postmeta')
                //         ->insert([
                //             'post_id' => $id,
                //             'meta_key' => 'ref_key',
                //             'meta_value' => $reference
                //         ]);
                //     }
                // }else{
                //     DB::table($this->table_prefix . 'postmeta')
                //     ->where('post_id', '=', $id)
                //     ->where('meta_key', '=', 'ref_key')
                //     ->delete();
                // }
                // END UPDATE REFERENCE KEY
            }
            if ($updated) {
                DB::table($this->table_prefix . 'posts')
                    ->where('ID', '=', $id)
                    ->update([
                        'post_modified' => date("Y-m-d H:i:s"),
                        'post_modified_gmt' => gmdate("Y-m-d H:i:s"),
                    ]);

                $this->broadcast();
            }
        }
    }

    private function save_carproof_url()
    {
        if ($this->serial_number) {
            $ch = curl_init(static::CARPROOF_ENDPOINT . '?id=' . static::CARPROOF_ID . '&vin=' . $this->serial_number);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_HEADER, 1);

            $response = curl_exec($ch);

            $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
            $rawHeader = substr($response, 0, $header_size);

            $headers = $this->headersToArray($rawHeader);
            if (isset($headers['location'])) {
                $this->save_meta('carproof_url', $headers['location']);
                return true;
            }
        }
        return false;
    }


    /**
     * Saves the WordPress taxonomy
     *
     * @return bool
     */
    private function save_taxonomy()
    {
        if (isset($this->ID)) {
            foreach (['fr', 'en'] as $lang) {
                $id = ($lang == 'en' ? $this->ID2 : $this->ID);

                $term = DB::table($this->table_prefix . 'terms')
                    ->where('slug', '=', $lang)
                    ->first();

                $term_taxonomy = DB::table($this->table_prefix . 'term_taxonomy')
                    ->where('term_id', '=', $term->term_id)
                    ->first();

                $term_relationship = DB::table($this->table_prefix . 'term_relationships')
                    ->where('object_id', '=', $id)
                    ->where('term_taxonomy_id', '=', $term_taxonomy->term_taxonomy_id)
                    ->first();

                if (empty($term_relationship)) {
                    DB::table($this->table_prefix . 'term_relationships')
                        ->insert([
                            'object_id' => $id,
                            'term_taxonomy_id' => $term_taxonomy->term_taxonomy_id
                        ]);
                }

                $pll_term = DB::table($this->table_prefix . 'terms')
                    ->where('name', '=', 'pll_' . md5("ctrlweb" . $this->ID . "--" . $this->ID2))
                    ->first();

                if (empty($pll_term)) {
                    $pll_term_id = DB::table($this->table_prefix . 'terms')
                        ->insertGetId([
                            'name'       => 'pll_' . md5("ctrlweb" . $this->ID . "--" . $this->ID2),
                            'slug'       => 'pll_' . md5("ctrlweb" . $this->ID . "--" . $this->ID2),
                            'term_group' => 0
                        ]);
                } else {
                    $pll_term_id = $pll_term->term_id;
                }

                $term_taxonomy = DB::table($this->table_prefix . 'term_taxonomy')
                    ->where('term_id', '=', $pll_term_id)
                    ->first();

                if (empty($term_taxonomy)) {
                    $term_taxonomy_id = DB::table($this->table_prefix . 'term_taxonomy')
                        ->insertGetId([
                            'term_id' => $pll_term_id,
                            'taxonomy' => 'post_translations',
                            'description' => serialize(['en' => $this->ID2, 'fr' => $this->ID]),
                            'parent' => 0,
                            'count' => 2,
                        ]);
                } else {
                    $term_taxonomy_id = $term_taxonomy->term_taxonomy_id;
                }

                $term_relationship = DB::table($this->table_prefix . 'term_relationships')
                    ->where('object_id', '=', $id)
                    ->where('term_taxonomy_id', '=', $term_taxonomy_id)
                    ->first();

                if (empty($term_relationship)) {
                    DB::table($this->table_prefix . 'term_relationships')
                        ->insert([
                            'object_id' => $id,
                            'term_taxonomy_id' => $term_taxonomy_id,
                            'term_order' => 0,
                        ]);
                }
            }


            return true;
        }

        return false;
    }


    /**
     * Broadcasts the addition or update of a vehicle in the WordPress database.
     */
    private function broadcast()
    {
        event(new InventoryUpdated($this));
    }


    /**
     * Helper function to sanitize title (taken from WordPress).
     *
     * @param $string
     *
     * @return mixed
     */
    private function remove_accents($string)
    {
        $string = str_replace('$', '', $string);
        $string = str_replace('+', '', $string);
        $string = str_replace('?', '', $string);
        $string = str_replace('.', '', $string);
        $string = str_replace(',', '-', $string);
        $string = str_replace('/', '-', $string);
        $string = str_replace('&', '-', $string);
        if (!preg_match('/[\x80-\xff]/', $string))
            return str_replace(' ', '-', strtolower($string));

        $chars = array(
            // Decompositions for Latin-1 Supplement
            'ª' => 'a', 'º' => 'o',
            'À' => 'A', 'Á' => 'A',
            'Â' => 'A', 'Ã' => 'A',
            'Ä' => 'A', 'Å' => 'A',
            'Æ' => 'AE', 'Ç' => 'C',
            'È' => 'E', 'É' => 'E',
            'Ê' => 'E', 'Ë' => 'E',
            'Ì' => 'I', 'Í' => 'I',
            'Î' => 'I', 'Ï' => 'I',
            'Ð' => 'D', 'Ñ' => 'N',
            'Ò' => 'O', 'Ó' => 'O',
            'Ô' => 'O', 'Õ' => 'O',
            'Ö' => 'O', 'Ù' => 'U',
            'Ú' => 'U', 'Û' => 'U',
            'Ü' => 'U', 'Ý' => 'Y',
            'Þ' => 'TH', 'ß' => 's',
            'à' => 'a', 'á' => 'a',
            'â' => 'a', 'ã' => 'a',
            'ä' => 'a', 'å' => 'a',
            'æ' => 'ae', 'ç' => 'c',
            'è' => 'e', 'é' => 'e',
            'ê' => 'e', 'ë' => 'e',
            'ì' => 'i', 'í' => 'i',
            'î' => 'i', 'ï' => 'i',
            'ð' => 'd', 'ñ' => 'n',
            'ò' => 'o', 'ó' => 'o',
            'ô' => 'o', 'õ' => 'o',
            'ö' => 'o', 'ø' => 'o',
            'ù' => 'u', 'ú' => 'u',
            'û' => 'u', 'ü' => 'u',
            'ý' => 'y', 'þ' => 'th',
            'ÿ' => 'y', 'Ø' => 'O',
            // Decompositions for Latin Extended-A
            'Ā' => 'A', 'ā' => 'a',
            'Ă' => 'A', 'ă' => 'a',
            'Ą' => 'A', 'ą' => 'a',
            'Ć' => 'C', 'ć' => 'c',
            'Ĉ' => 'C', 'ĉ' => 'c',
            'Ċ' => 'C', 'ċ' => 'c',
            'Č' => 'C', 'č' => 'c',
            'Ď' => 'D', 'ď' => 'd',
            'Đ' => 'D', 'đ' => 'd',
            'Ē' => 'E', 'ē' => 'e',
            'Ĕ' => 'E', 'ĕ' => 'e',
            'Ė' => 'E', 'ė' => 'e',
            'Ę' => 'E', 'ę' => 'e',
            'Ě' => 'E', 'ě' => 'e',
            'Ĝ' => 'G', 'ĝ' => 'g',
            'Ğ' => 'G', 'ğ' => 'g',
            'Ġ' => 'G', 'ġ' => 'g',
            'Ģ' => 'G', 'ģ' => 'g',
            'Ĥ' => 'H', 'ĥ' => 'h',
            'Ħ' => 'H', 'ħ' => 'h',
            'Ĩ' => 'I', 'ĩ' => 'i',
            'Ī' => 'I', 'ī' => 'i',
            'Ĭ' => 'I', 'ĭ' => 'i',
            'Į' => 'I', 'į' => 'i',
            'İ' => 'I', 'ı' => 'i',
            'Ĳ' => 'IJ', 'ĳ' => 'ij',
            'Ĵ' => 'J', 'ĵ' => 'j',
            'Ķ' => 'K', 'ķ' => 'k',
            'ĸ' => 'k', 'Ĺ' => 'L',
            'ĺ' => 'l', 'Ļ' => 'L',
            'ļ' => 'l', 'Ľ' => 'L',
            'ľ' => 'l', 'Ŀ' => 'L',
            'ŀ' => 'l', 'Ł' => 'L',
            'ł' => 'l', 'Ń' => 'N',
            'ń' => 'n', 'Ņ' => 'N',
            'ņ' => 'n', 'Ň' => 'N',
            'ň' => 'n', 'ŉ' => 'n',
            'Ŋ' => 'N', 'ŋ' => 'n',
            'Ō' => 'O', 'ō' => 'o',
            'Ŏ' => 'O', 'ŏ' => 'o',
            'Ő' => 'O', 'ő' => 'o',
            'Œ' => 'OE', 'œ' => 'oe',
            'Ŕ' => 'R', 'ŕ' => 'r',
            'Ŗ' => 'R', 'ŗ' => 'r',
            'Ř' => 'R', 'ř' => 'r',
            'Ś' => 'S', 'ś' => 's',
            'Ŝ' => 'S', 'ŝ' => 's',
            'Ş' => 'S', 'ş' => 's',
            'Š' => 'S', 'š' => 's',
            'Ţ' => 'T', 'ţ' => 't',
            'Ť' => 'T', 'ť' => 't',
            'Ŧ' => 'T', 'ŧ' => 't',
            'Ũ' => 'U', 'ũ' => 'u',
            'Ū' => 'U', 'ū' => 'u',
            'Ŭ' => 'U', 'ŭ' => 'u',
            'Ů' => 'U', 'ů' => 'u',
            'Ű' => 'U', 'ű' => 'u',
            'Ų' => 'U', 'ų' => 'u',
            'Ŵ' => 'W', 'ŵ' => 'w',
            'Ŷ' => 'Y', 'ŷ' => 'y',
            'Ÿ' => 'Y', 'Ź' => 'Z',
            'ź' => 'z', 'Ż' => 'Z',
            'ż' => 'z', 'Ž' => 'Z',
            'ž' => 'z', 'ſ' => 's',
            // Decompositions for Latin Extended-B
            'Ș' => 'S', 'ș' => 's',
            'Ț' => 'T', 'ț' => 't',
            // Euro Sign
            '€' => 'E',
            // GBP (Pound) Sign
            '£' => '',
            // Vowels with diacritic (Vietnamese)
            // unmarked
            'Ơ' => 'O', 'ơ' => 'o',
            'Ư' => 'U', 'ư' => 'u',
            // grave accent
            'Ầ' => 'A', 'ầ' => 'a',
            'Ằ' => 'A', 'ằ' => 'a',
            'Ề' => 'E', 'ề' => 'e',
            'Ồ' => 'O', 'ồ' => 'o',
            'Ờ' => 'O', 'ờ' => 'o',
            'Ừ' => 'U', 'ừ' => 'u',
            'Ỳ' => 'Y', 'ỳ' => 'y',
            // hook
            'Ả' => 'A', 'ả' => 'a',
            'Ẩ' => 'A', 'ẩ' => 'a',
            'Ẳ' => 'A', 'ẳ' => 'a',
            'Ẻ' => 'E', 'ẻ' => 'e',
            'Ể' => 'E', 'ể' => 'e',
            'Ỉ' => 'I', 'ỉ' => 'i',
            'Ỏ' => 'O', 'ỏ' => 'o',
            'Ổ' => 'O', 'ổ' => 'o',
            'Ở' => 'O', 'ở' => 'o',
            'Ủ' => 'U', 'ủ' => 'u',
            'Ử' => 'U', 'ử' => 'u',
            'Ỷ' => 'Y', 'ỷ' => 'y',
            // tilde
            'Ẫ' => 'A', 'ẫ' => 'a',
            'Ẵ' => 'A', 'ẵ' => 'a',
            'Ẽ' => 'E', 'ẽ' => 'e',
            'Ễ' => 'E', 'ễ' => 'e',
            'Ỗ' => 'O', 'ỗ' => 'o',
            'Ỡ' => 'O', 'ỡ' => 'o',
            'Ữ' => 'U', 'ữ' => 'u',
            'Ỹ' => 'Y', 'ỹ' => 'y',
            // acute accent
            'Ấ' => 'A', 'ấ' => 'a',
            'Ắ' => 'A', 'ắ' => 'a',
            'Ế' => 'E', 'ế' => 'e',
            'Ố' => 'O', 'ố' => 'o',
            'Ớ' => 'O', 'ớ' => 'o',
            'Ứ' => 'U', 'ứ' => 'u',
            // dot below
            'Ạ' => 'A', 'ạ' => 'a',
            'Ậ' => 'A', 'ậ' => 'a',
            'Ặ' => 'A', 'ặ' => 'a',
            'Ẹ' => 'E', 'ẹ' => 'e',
            'Ệ' => 'E', 'ệ' => 'e',
            'Ị' => 'I', 'ị' => 'i',
            'Ọ' => 'O', 'ọ' => 'o',
            'Ộ' => 'O', 'ộ' => 'o',
            'Ợ' => 'O', 'ợ' => 'o',
            'Ụ' => 'U', 'ụ' => 'u',
            'Ự' => 'U', 'ự' => 'u',
            'Ỵ' => 'Y', 'ỵ' => 'y',
            // Vowels with diacritic (Chinese, Hanyu Pinyin)
            'ɑ' => 'a',
            // macron
            'Ǖ' => 'U', 'ǖ' => 'u',
            // acute accent
            'Ǘ' => 'U', 'ǘ' => 'u',
            // caron
            'Ǎ' => 'A', 'ǎ' => 'a',
            'Ǐ' => 'I', 'ǐ' => 'i',
            'Ǒ' => 'O', 'ǒ' => 'o',
            'Ǔ' => 'U', 'ǔ' => 'u',
            'Ǚ' => 'U', 'ǚ' => 'u',
            // grave accent
            'Ǜ' => 'U', 'ǜ' => 'u',
        );

        $string = str_replace(' ', '-', strtolower(strtr($string, $chars)));

        return $string;
    }
    
    private function removePlusSign($string)
    {
        $string = str_replace(' +', '', $string);
        return $string;
    }

    private function headersToArray($rawHeaders)
    {
        $headers = array();

        foreach (explode("\r\n", $rawHeaders) as $i => $line) {
            if ($i === 0) {
                $headers['http_code'] = $line;
            } else {
                if (count(explode(': ', $line)) === 1) {
                    continue;
                }
                list($key, $value) = explode(': ', $line);

                $headers[$key] = $value;
            }
        }

        return $headers;
    }
}
